@@ -24,7 +24,7 @@ read.aggregation <- function(filename, mnemonics = NULL, ...,
   }
 
   data <- melt.aggregation(data.raw)
-  
+
   data$label <- relevel.mnemonics(data$label, mnemonics)
 
   ## Order components by the order observed in the file
@@ -352,6 +352,7 @@ xyplot.aggregation <- function(agg.data = NULL,
                else list(points = FALSE, lines = TRUE),
     strip = strip.custom(horizontal = FALSE),
     strip.height = 10,
+    par.strip.text <- list(cex = 0.7),
     xlab = NULL,
     ylab = if (normalize)
               expression(paste("Enrichment: ",
@@ -422,7 +423,8 @@ xyplot.aggregation <- function(agg.data = NULL,
                auto.key = auto.key, as.table = TRUE, strip = strip,
                xlab = xlab, ylab = ylab, sub = sub,
                signif = data$signif, ngroups = ngroups,
-               panel = "panel.superpose", panel.groups = panel, ...)
+               panel = "panel.superpose", panel.groups = panel,
+               par.strip.text = par.strip.text, ...)
 
   trellis.raw <- do.call(xyplot, args)
 
@@ -470,7 +472,7 @@ save.gene.aggregations <- function(dirpath, namebase1, namebase2, tabfilename,
   mnemonics <- read.mnemonics(mnemonic_file)
   data <- read.aggregation(tabfilename, mnemonics = mnemonics,
                            comment.char = comment.char)
-  
+
   image.size <- margin.size +
     panel.size * ceiling(sqrt(nlevels(data$data$label) / 2))
   ngroup1 <- as.integer(data$metadata[["spacers"]])
