@@ -31,7 +31,9 @@ read.overlap <- function(filename, mnemonics = NULL, col_mnemonics = NULL,
     res[, 2:ncol(res)] <- apply(res[, 2:ncol(res)], 2, as.numeric)
   }
   
-  res
+  metadata <- read.metadata(filename, comment.char = comment.char)
+
+  list(data = res, metadata = metadata)
 }
 
 # Given a list of labels, returns a list of colors to give those labels
@@ -74,112 +76,120 @@ label.colors <-
 }
 
 panel.overlap <-
-  function(x, y, groups, subscripts, labels, colors, reference, 
-           plot.text = TRUE, plot.points = FALSE, significances = NULL, ...)
+  function(x, y, groups, subscripts, labels, colors,
+           plot.text = TRUE, plot.points = TRUE, cumulative = TRUE, ...)
 {
-  ## Plot x == y reference line
-  ## panel.abline(c(0, 1), reference = TRUE, ...)
-
   if (plot.points) {
-    panel.xyplot(x, y, groups = groups, col = colors, 
+    panel.xyplot(x, y, pch = 20, col = "black",
                  subscripts = subscripts, ...)
   }
 
   if (plot.text) {
     if (plot.points) {
-      pos <- 1
-      offset <- 1
+      adj <- c(-0.25, -0.25)
     } else {
-      pos <- NULL
-      offset <- NULL
+      adj <- NULL
+    }
+    labels <- as.character(labels)
+    if (cumulative) {
+      ## Prepend a "+" to all but the first label
+      x.ord <- order(x)
+      indices <- x.ord[2:length(x.ord)]
+      labels[indices] <- paste("+", labels[indices], sep = "")
     }
     panel.text(x, y, 
                labels = labels,
                col = colors,
-               pos = pos,
-               offset = offset,
+               adj = adj,
                #cex = c(0.2, 0.9),
                ...)
   }
-
-  # Plot significances next to points
-  if (! is.null(significances)) {
-    significant <- significances < 0.05
-    panel.text(x[significant], y[significant],
-               labels = format(significances[significant], digits = 1),
-               col = "black",
-               pos = 4,
-               #cex = 0.8,
-               offset = 1,
-               ...)
-  }
 }
 
-# Returns a data table of tp/tn/fp/fn for the given counts
-overlap.stats <-
-  function(counts)
-{
+# Returns a data table of tp/fp/fn for the given counts
+overlap.stats <- function(counts, cumulative = TRUE) {
+  labels <- counts$label
   total.counts <- counts$total
   none.counts <- counts$none
   feature.counts <- subset(counts, select = -c(label, total, none))
-  total.sum <- sum(as.numeric(total.counts))
   feature.sums <- colSums(feature.counts)
+#  total.sum <- sum(as.numeric(total.counts))
+
   tp <- feature.counts
   fp <- total.counts - feature.counts
   fn <- t(feature.sums - t(feature.counts))
-  tn <- total.sum - total.counts - fn
+#  tn <- total.sum - total.counts - fn
 
-  res <- list()
-  res$label <- counts$label
-  res$tp <- tp
-  res$tn <- tn
-  res$fp <- fp
-  res$fn <- fn
-  res$col.names <- c("label",
-                     paste("tp", colnames(tp), sep = "."),
-                     paste("tn", colnames(tn), sep = "."),
-                     paste("fp", colnames(fp), sep = "."),
-                     paste("fn", colnames(fn), sep = "."))
+  if (cumulative) {
+    precision <- tp / (tp + fp)
+    ## Replace columns with cumulative data (sorted by precision)
+    for (col.i in 1:ncol(precision)) {
+      label.ord <- order(precision[, col.i], decreasing = TRUE)
+      ## Calculate cumulative statistics
+      total.cum <- cumsum(total.counts[label.ord])
+      tp.cum <- cumsum(feature.counts[label.ord, col.i])
+      fp.cum <- total.cum - tp.cum
+      fn.cum <- feature.sums[col.i] - tp.cum
+      tp[label.ord, col.i] <- tp.cum
+      fp[label.ord, col.i] <- fp.cum
+      fn[label.ord, col.i] <- fn.cum
+    }
+  }
+  
+  res <- list(label = labels,
+              tp = tp,
+              fp = fp,
+              fn = fn)
                      
   res
 }
 
-# Convert the stats to a single data frame (suitable for write.stats)
-stat.data.frame <-
-  function(stats)
-{
-  stat.df <- data.frame(stats$label, stats$tp, stats$tn, stats$fp, stats$fn)
-  colnames(stat.df) <- stats$col.names
+# Returns a data frame of p/r for the given counts
+pr.stats <- function(data, ...) {
+  stats <- overlap.stats(data$data, ...)
+
+  precision <- suppressMessages(melt(with(stats, tp / (tp + fp))))
+  recall <- suppressMessages(melt(with(stats, tp / (tp + fn))))
+
+  res <- data.frame(label = stats$label, group = precision$variable,
+                    precision = precision$value, recall = recall$value)
   
-  stat.df
+  list(data = res, metadata = data$metadata)
 }
 
-# Write a stat data frame to a file
-write.stats <-
-  function(tabbasename, dirpath = ".", stat.df)
-{
-  tabfilename <- extpaste(tabbasename, "tab")
+# Write an overlap data frame to a file
+write.stats <- function(stats, namebase, dirpath = ".", clobber = FALSE) {
+  data <- stats$data
+  tabfilename <- extpaste(namebase, "tab")
   tabfilepath <- file.path(dirpath, tabfilename)
-  
-  write.table(stat.df, tabfilepath, quote = FALSE, sep = "\t", row.names = FALSE)
+
+  if (file.exists(tabfilepath) && !clobber) {
+    stop("Found stats table: ", tabfilepath,
+         ". Use --clobber to overwrite!", sep = "")
+  }
+  row.ord = order(data$group, data$precision, decreasing = TRUE)
+  data.formatted <- format(data[row.ord,], digits = 3)
+  write.table(data.formatted, tabfilepath, quote = FALSE,
+              sep = "\t", row.names = FALSE)
 }
 
 xyplot.overlap <-
-  function(data,
-           metadata = NULL,
-#           x = tpr ~ fpr | factor, 
-           x = precision ~ recall | factor, 
+  function(overlap.data = NULL,
+           data = overlap.data$data,
+           metadata = overlap.data$metadata,
+           mode = metadata$mode,
+           x = precision ~ recall | group, 
            small.cex = 1.0,
            large.cex = 1.0,
            as.table = TRUE,
-           aspect = "fill",
            auto.key = FALSE, #list(space = "right"),
-#           xlab = list("False positive rate (FP / (FP + TN))", cex = large.cex),
-#           ylab = list("True positive rate (TP / (TP + FN))", cex = large.cex),
            xlab = list("Recall (TP / (TP + FN))", cex = large.cex),
            ylab = list("Precision (TP / (TP + FP))", cex = large.cex),
+           cumulative = TRUE,
+           sub = if (cumulative) "Cumulative P-R (left-to-right, by precision)"
+                 else NULL,
            x.lim = c(0, 1),
-           y.lim = c(0, 1),
+           y.lim = NULL,
            scales = list(cex = small.cex,
              x = list(limits = x.lim),
              y = list(limits = y.lim)),
@@ -189,18 +199,27 @@ xyplot.overlap <-
            par.strip.text = list(cex = small.cex),
            ...)
 {
-  stats <- overlap.stats(data)
-  precision <- suppressMessages(melt(stats$tp / (stats$tp + stats$fp)))
-  recall <- suppressMessages(melt(stats$tp / (stats$tp + stats$fn)))
-  stats.merged <- data.frame(label = labels, factor = precision$variable,
-                            precision = precision$value, recall = recall$value)
-
-  xyplot(x, stats.merged, groups = label,
+  if (mode != "bases") {
+    warning("P-R performance plot only mathematically valid for base-overlap")
+  }
+  if (is.null(y.lim)) {
+    y.max <- max(data$precision)
+    y.lim <-
+      if (y.max >= 0.5) {
+        c(0, 1)
+      } else {
+        c(0, y.max * 1.05)
+      }
+  }
+  
+  xyplot(x, data, groups = label,
          as.table = as.table, 
-         aspect = aspect,
          auto.key = auto.key,
-         xlab = xlab, ylab = ylab,
+         xlab = xlab,
+         ylab = ylab,
+         sub = sub,
          scales = scales,
+         cumulative = cumulative,
          panel = panel,
          labels = labels,
          colors = colors,
@@ -208,125 +227,56 @@ xyplot.overlap <-
          ...)
 }
 
-plot.overlap <-  function(tabfile, mnemonics = NULL, col_mnemonics = NULL,
-                          comment.char = "#", ...) {
+plot.overlap.performance <-  function(tabfile, mnemonics = NULL,
+                                      col_mnemonics = NULL,
+                                      cumulative = TRUE,
+                                      comment.char = "#", ...) {
   ## Plot the predictive ability of each segment label for each feature class
-  ##   in ROC space
+  ##   in PR space
   ##
   ## tabfile: a tab file containing overlap data with segment labels on the rows
   ##   and feature classes on the columns and the overlap "count" at the
   ##   intersection. Row and columns should have labels.
   
-  counts <- read.overlap(tabfile, mnemonics = mnemonics,
-                         col_mnemonics = col_mnemonics,
-                         comment.char = comment.char)
+  data <- read.overlap(tabfile, mnemonics = mnemonics,
+                       col_mnemonics = col_mnemonics,
+                       comment.char = comment.char)
+  stats <- pr.stats(data, cumulative = cumulative)
   
-  #stat.df <- stat.data.frame(stats)
-  #if (!is.null(basename)) {
-  #  write.stats(tabbasename = basename, dirpath = dirpath, stat.df)
-  #}
-
-  xyplot.overlap(stats, ...)
+  xyplot.overlap(stats, cumulative = cumulative, ...)
 }
 
-save.overlap <- function(dirpath, namebase, tabfilename,
-                         mnemonic_file = NULL,
-                         col_mnemonic_file = NULL,
-                         clobber = FALSE,
-                         panel.size = 300,  # px
-                         comment.char = "#",
-                         ...) {
+save.overlap.performance <- function(dirpath, namebase, tabfilename,
+                                     mnemonic_file = NULL,
+                                     col_mnemonic_file = NULL,
+                                     clobber = FALSE,
+                                     cumulative = TRUE,
+                                     panel.size = 300,  # px
+                                     comment.char = "#",
+                                     ...) {
   mnemonics <- read.mnemonics(mnemonic_file)
   col.mnemonics <- read.mnemonics(col_mnemonic_file)
   data <- read.overlap(tabfilename, mnemonics = mnemonics,
                        col_mnemonics = col.mnemonics,
                        comment.char = comment.char)
-  metadata <- read.metadata(tabfilename, comment.char = comment.char)
-
-  panels.sqrt <- max(c(sqrt(nlevels(data$label)), 1))
+  stats <- pr.stats(data, cumulative = cumulative)
+  write.stats(stats, namebase, dirpath, clobber = clobber)
+  
+  panels.sqrt <- max(c(sqrt(nlevels(stats$data$group)), 1))
   width <- 100 + panel.size * ceiling(panels.sqrt)
   height <- 200 + panel.size * floor(panels.sqrt)
   save.images(dirpath, namebase,
-              xyplot.overlap(data, metadata = metadata, ...),
+              xyplot.overlap(stats, cumulative = cumulative, ...),
               width = width,
               height = height,
               clobber = clobber)
 }
 
-############### P-VALUE ANALYSIS ############
-
-read.pvalues <-  function(...) {
-  read.overlap(...)
-}
-
-panel.pvalues <- function(x, y, subscripts = NULL, groups = NULL,
-                          reference = 0.01, col = NULL, ...)
-{
-  ref.x <- log10(reference)
-  panel.refline(v = ref.x)
-  out.of.bounds = is.infinite(x)
-  x[out.of.bounds] <- min(x[!out.of.bounds]) * 2
-
-  panel.barchart(x, y, subscripts = subscripts, groups = groups,
-                 stacked = FALSE, col = col[y], ...)
-}
-
-barchart.pvalues <- function(data,
-                             x = reorder(label, -value) ~ value,
-                             groups = if (ngroups > 1) variable else NULL,
-                             as.table = TRUE,
-                             main = "Approximate significance of overlap",
-                             panel = panel.pvalues,
-                             xlab = "p-value",
-                             ylab = "Segment label",
-                             reference = 0.01,
-                             origin = 0,
-                             auto.key = if (ngroups > 1) list(points = FALSE)
-                                        else FALSE,
-                             colors = label.colors(data.melted$label),
-                             scales = list(x = list(log = TRUE)),
-                             ...)
-{
-  ## Create a barchart from overlap pvalue data
-  ##
-  ## data: a data frame containing pvalue data
-  
-  data.melted <- melt(data, id.vars = "label")
-  ngroups = nlevels(data.melted$variable)
-  
-  colors.ordered <- colors[order(-data.melted$value)]
-  xyplot(x = x,
-         data = data.melted,
-         groups = groups,
-         panel = panel,
-         as.table = as.table,
-         reference = reference,
-         main = main,
-         col = colors.ordered,
-         scales = scales,
-         origin = origin,
-         auto.key = auto.key,
-         xlab = xlab,
-         ylab = ylab,
-         #par.settings = list(clip = list(panel = "off")),
-         ...)
-}
-
-plot.overlap.pvalues <- function(tabfile, mnemonics = NULL,
-                                 col_mnemonics = NULL,
-                                 comment.char = "#", ...) {
-  ## Plot the overlap pvalue data
-  pvalues <- read.pvalues(tabfile, mnemonics = mnemonics,
-                          col_mnemonics = col_mnemonics,
-                          comment.char = comment.char)
-  barchart.pvalues(pvalues, ...)
-}
-
-
 ############### OVERLAP HEATMAP ############
 
-levelplot.overlap <- function(data,
-                              metadata = NULL,
+levelplot.overlap <- function(overlap.data = NULL,
+                              data = overlap.data$data,
+                              metadata = overlap.data$metadata,
                               mode = metadata[["mode"]],
                               row.normalize = TRUE,
                               y.mode = if (row.normalize) "Fraction"
@@ -341,6 +291,7 @@ levelplot.overlap <- function(data,
                               cluster.rows = cluster,
                               cluster.cols = cluster,
                               num.colors = 100,
+                              aspect = "fill",
                               max.contrast = FALSE,  # Saturate color range
                               col.range = if (max.contrast) NULL else c(0, 1),
                               scales = list(x = list(rot = 90)),
@@ -407,6 +358,7 @@ levelplot.overlap <- function(data,
             ylab = ylab,
             cuts = cuts,
             scales = scales,
+            aspect = aspect,
             at = colorkey.at,
             col.regions = palette,
             ...)
@@ -424,9 +376,8 @@ plot.overlap.heatmap <- function(filename, mnemonics = NULL,
   data <- read.overlap(filename, mnemonics = mnemonics,
                        col_mnemonics = col_mnemonics,
                        comment.char = comment.char)
-  metadata <- read.metadata(filename, comment.char = comment.char)
 
-  levelplot.overlap(data, metadata = metadata, ...)
+  levelplot.overlap(data, ...)
 }
 
 save.overlap.heatmap <- function(dirpath, namebase, tabfilename,
@@ -441,12 +392,11 @@ save.overlap.heatmap <- function(dirpath, namebase, tabfilename,
   data <- read.overlap(tabfilename, mnemonics = mnemonics,
                        col_mnemonics = col.mnemonics,
                        comment.char = comment.char)
-  metadata <- read.metadata(tabfilename, comment.char = comment.char)
 
-  height <- 400 + panel.size * nrow(data)
-  width <- 400 + panel.size * ncol(data)
+  height <- 400 + panel.size * nrow(data$data)
+  width <- 400 + panel.size * ncol(data$data)
   save.images(dirpath, namebase,
-              levelplot.overlap(data,  metadata = metadata, ...),
+              levelplot.overlap(data, ...),
               height = height,
               width = width,
               clobber = clobber)
