@@ -14,7 +14,7 @@ as provided by exporting UCSC gene data in GTF format.
 # A package-unique, descriptive module name used for filenames, etc
 MODULE="feature_aggregation"
 
-__version__ = "$Revision: 280 $"
+__version__ = "$Revision: 292 $"
 
 import os
 import sys
@@ -693,7 +693,11 @@ def save_tab(segmentation, features, counts, components, component_bins,
                         count += len(chrom_features)
                     elif component in GENE_COMPONENTS:
                         # Separate by component (key column)
-                        label_key = features.label_key(component)
+                        try:
+                            label_key = features.label_key(component)
+                        except KeyError:
+                            continue
+
                         keep = chrom_features['key'] == label_key
                         sum += (chrom_features['end'][keep] -
                                 chrom_features['start'][keep]).sum()
