@@ -13,152 +13,20 @@ BED3+ or GFF format.
 
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 201 $"
+__version__ = "$Revision: 228 $"
 
 
 import os
 import sys
 
-from numpy import array, NaN, ndarray
-from warnings import warn
+from numpy import array, NaN
 
-from . import Segmentation
-from .common import load_features
-
-EXT_BED = "bed"
-EXT_GFF = "gff"
-EXT_GTF = "gtf"
-EXT_GZ = "gz"
+from . import Annotations, log, Segmentation
+from .common import FeatureScanner
 
 STANDARD_FIELDS = ["chrom", "start", "end", "name"]
 DELIM = "\t"
 
-class FeatureScanner(object):
-    def __init__(self, features):
-        self._current = None
-        self._nearest = None
-        if isinstance(features, ndarray):
-            self._iter = iter(features)
-            try:
-                self._current = self._iter.next()
-            except StopIteration:
-                pass
-            else:
-                self._nearest = self._current
-        else:
-            warn("Feature list not of understood type. Treated as empty")
-
-        self._has_next = (self._current is not None)
-        self._overlapping = []
-
-        self._prev_best = None
-        self._next_best = None
-        self._prev_start = None
-
-    @staticmethod
-    def distance(feature1, feature2):
-        """Return the distance between two features (zero if overlap)"""
-        if feature1['end'] <= feature2['start']:
-            return feature2['start'] - feature1['end'] + 1
-        elif feature2['end'] <= feature1['start']:
-            return feature1['start'] - feature2['end'] + 1
-        else:  # Features overlap
-            return 0
-
-    @staticmethod
-    def are_overlapping(feature1, feature2):
-        """Return the whether or not two features overlap"""
-        return feature1['start'] < feature2['end'] and \
-                feature2['start'] < feature1['end']
-
-    def _seek(self, segment):
-        # Seek internal records up to segment, updated self._overlapping
-        # and self._nearest
-
-        # Clear out any non-overlapping features from self._overlapping
-        if self._overlapping:
-            i = 0
-            while i < len(self._overlapping):
-                if self.are_overlapping(segment, self._overlapping[i]):
-                    i += 1
-                else:
-                    del self._overlapping[i]
-
-        # If any are overlapping, nearest is 0 distance
-        if self._overlapping:
-            nearest_dist = 0
-            self._nearest = self._overlapping[0]
-        elif self._nearest is not None:
-            # Closest feature will start as the closest from the last iteration
-            nearest_dist = self.distance(self._nearest, segment)
-        else:
-            assert not self._has_next
-
-        # Scan through features until we get past segment
-        if self._has_next:
-            segment_start = segment['start']
-            segment_end = segment['end']
-            while self._current['start'] < segment_end:
-                # dist will be negative if this overlaps
-                dist = segment_start - self._current['end'] + 1
-                if dist < 0:
-                    self._overlapping.append(self._current)
-
-                if dist < nearest_dist:
-                    self._nearest = self._current
-                    nearest_dist = dist
-
-                try:
-                    self._current = self._iter.next()
-                except StopIteration:
-                    self._has_next = False
-                    break
-
-            # Check distance of first feature past segment as well
-            dist = self._current['start'] - segment_end + 1
-            if dist < nearest_dist:
-                self._nearest = self._current
-                nearest_dist = dist
-
-    def nearest(self, segment):
-        """Return a list of the nearest features for the next segment
-
-        This will contain a single feature if no features overlap the given
-        segment, else it will be a list of features that overlap.
-
-        If no features were found at all, this will be None.
-
-        Should be called with in-order segments
-
-        """
-        self._seek(segment)
-        if self._overlapping:
-            return self._overlapping
-        elif self._nearest is None:
-            return None
-        else:
-            return [self._nearest]
-
-def get_file_ext(filename):
-    # Ignore g'zipping
-    head, tail = os.path.splitext(os.path.basename(filename))
-    if tail == os.extsep + EXT_GZ:
-        head, tail = os.path.splitext(head)
-
-    if tail:
-        return tail[1:]  # Drop leading "."
-    else:
-        return None  # Didn't find ext
-
-def load_data(filename, verbose=False):
-    ext = get_file_ext(filename)
-    if ext == EXT_BED:
-        return Segmentation(filename, verbose=verbose)
-    elif ext == EXT_GFF or ext == EXT_GTF:
-        return load_features(filename, verbose=verbose)
-    else:
-        raise NotImplementedError("Unexpected file type: %s" % ext)
-
 def print_header_line(feature_files):
     file_fields = [os.path.basename(filename) for filename in feature_files]
     fields = STANDARD_FIELDS + file_fields
@@ -204,7 +72,7 @@ def indices_to_bools(indices, length):
 
     return bools
 
-def feature_distance(segment_file, feature_files, verbose=False,
+def feature_distance(segment_file, feature_files, verbose=True,
                      correct_strands=[], print_nearest=[], all_overlapping=[]):
     # Lists are of integer file indices (0-indexed), for which files that
     # property applies to.
@@ -224,19 +92,18 @@ def feature_distance(segment_file, feature_files, verbose=False,
         die("File index for all overlapping: %s is invalid" % e)
 
     # Load segment file
-    segment_obj = load_data(segment_file, verbose=verbose)
+    segment_obj = Segmentation(segment_file, verbose=verbose)
     labels = segment_obj.labels
     segment_data = segment_obj.chromosomes
 
     # Load feature files
-    feature_objs = [load_data(feature_file, verbose=verbose)
+    feature_objs = [Annotations(feature_file, verbose=verbose)
                     for feature_file in feature_files]
     feature_datas = [feature_obj.chromosomes for feature_obj in feature_objs]
 
     print_header_line(feature_files)
     for chrom in segment_data:
-        if verbose:
-            print >>sys.stderr, "%s" % chrom
+        log("%s" % chrom, verbose)
 
         segments = segment_data[chrom]
         features_list = []
@@ -252,16 +119,20 @@ def feature_distance(segment_file, feature_files, verbose=False,
         for segment in segments:
             distances = []
             for file_i, feature_scanner in enumerate(feature_scanners):
-                features = feature_scanner.nearest(segment)
+                nearest = feature_scanner.nearest(segment)
                 distance = NaN
-                if features is not None:
-                    # If we are only printing one, use the first regardless
-                    if not all_overlapping[file_i]:
-                        features = features[0:1]
+                if nearest is not None:
+                    if isinstance(nearest, list):
+                        # If we are only printing one, use the first regardless
+                        if not all_overlapping[file_i]:
+                            nearest = nearest[0:1]
+                    else:
+                        # Make sure nearest is a list, even if only one element
+                        nearest = [nearest]
 
                     # Save multiple distances as a list of them
                     distance = []
-                    for feature in features:
+                    for feature in nearest:
                         one_dist = FeatureScanner.distance(segment, feature)
                         if correct_strand[file_i]:
                             try:
@@ -295,15 +166,15 @@ def feature_distance(segment_file, feature_files, verbose=False,
 def parse_options(args):
     from optparse import OptionParser
 
-    usage = "%prog [OPTIONS] SEGMENTFILE FEATUREFILE..."
+    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATIONS..."
     version = "%%prog %s" % __version__
 
     parser = OptionParser(usage=usage, version=version,
                           description=__doc__.strip())
 
-    parser.add_option("-v", "--verbose", dest="verbose",
-                      default=False, action="store_true",
-                      help="Print diagnostic messages")
+    parser.add_option("-q", "--quiet", dest="verbose",
+                      default=True, action="store_false",
+                      help="Do not print diagnostic messages.")
     parser.add_option("-s", "--strand-correct", dest="correct_strands",
                       default=[], action="append", metavar="N", type="int",
                       help="Strand correct features from the Nth feature file"
